@@ -34,19 +34,33 @@
 #ifndef _CPP_BITS_CHAR_TRAITS_H
 #define _CPP_BITS_CHAR_TRAITS_H 1
 
-#include <bits/fpos.h>
+#include <bits/std_cwchar.h> 	// For mbstate_t.
+#include <bits/std_cstring.h> 	// For memmove, memset, memchr
+#include <bits/fpos.h> 		// For streamoff, streamsize
 
 namespace std {
 
+  // Same as iosfwd
+#ifdef _GLIBCPP_RESOLVE_LIB_DEFECTS
+  // Can't have self-recursive types for streampos. 
+  // 21.1.3.1 char_traits sets size_type to streampos
+  // 27.4.1 
+  // And here, where streampos is typedefed to fpos<traits::state_type>
+    typedef fpos<mbstate_t> 	streampos;
+#  ifdef _GLIBCPP_USE_WCHAR_T
+    typedef fpos<mbstate_t> 	wstreampos;
+#  endif
+#endif
+
   // 21.1.2 Basis for explicit _Traits specialization 
-  // NB: that for any given actual character type this definition is
+  // NB: That for any given actual character type this definition is
   // probably wrong.
 
   template<class _CharT>
     struct char_traits
     {
-      typedef _CharT char_type;
-      typedef _CharT int_type;
+      typedef _CharT 	char_type;
+      typedef long  	int_type;
       typedef streampos pos_type;
       typedef streamoff off_type;
       typedef mbstate_t state_type;
@@ -64,15 +78,15 @@ namespace std {
       { return __c1 < __c2; }
 
       static int 
-      compare(const char_type* __s1, const char_type* __s2, size_t __n)
+      compare(const char_type* __s1, const char_type* __s2, int_type __n)
       { 
-	for (size_t __i = 01; __i < __n; ++__i)
+	for (int_type __i = 0; __i < __n; ++__i)
 	  if (!eq(__s1[__i],__s2[__i]))
 	    return lt(__s1[__i],__s2[__i]) ? -1 : 1;
 	return 0; 
       }
 
-      static size_t 
+      static size_t
       length(const char_type* __s)
       { 
 	const char_type* __p = __s; 
@@ -89,52 +103,52 @@ namespace std {
       }
 
       static char_type* 
-      move(char_type* __s1, const char_type* __s2, size_t __n)
-      { return (char_type*) memmove (__s1, __s2, __n*sizeof (char_type)); }
+      move(char_type* __s1, const char_type* __s2, int_type __n)
+      { return (char_type*) memmove(__s1, __s2, __n * sizeof(char_type)); }
 
       static char_type* 
-      copy(char_type* __s1, const char_type* __s2, size_t __n)
-      { return (char_type*) memcpy (__s1, __s2, __n*sizeof (char_type)); }
+      copy(char_type* __s1, const char_type* __s2, int_type __n)
+      { return (char_type*) memcpy(__s1, __s2, __n * sizeof(char_type)); }
 
       static char_type* 
-      assign(char_type* __s, size_t __n, char_type __a)
+      assign(char_type* __s, int_type __n, char_type __a)
       { 
 	for (char_type* __p = __s; __p - __s < __n; ++__p) 
-	  assign(*__p,__a);
+	  assign(*__p, __a);
         return __s; 
       }
 
       static int_type 
       not_eof(const int_type& __c)
-      { return eq_int_type (__c,eof ()) ? int_type(0) : __c; }
+      { return eq_int_type(__c,eof()) ? int_type(0) : __c; }
 
       static char_type 
       to_char_type(const int_type& __c)
-      { return char_type (__c); }
+      { return char_type(__c); }
 
       static int_type 
-      to_int_type(const char_type& __c) { return int_type (__c); }
+      to_int_type(const char_type& __c) { return int_type(__c); }
 
       static bool 
       eq_int_type(const int_type& __c1, const int_type& __c2)
       { return __c1 == __c2; }
 
       static state_type 
-      get_state (pos_type __pos) { return __pos.state (); }
+      get_state (pos_type __pos) { return __pos.state(); }
 
       static int_type 
-      eof() { return int_type (-1); }
+      eof() { return int_type(-1); }
 
       static int_type 
-      eos() { return int_type (); }
+      eos() { return int_type(); }
     };
 
   // 21.1.4  char_traits specializations
   template<>
     struct char_traits<char>
     {
-      typedef char char_type;
-      typedef int int_type;
+      typedef char 	char_type;
+      typedef int 	int_type;
       typedef streampos pos_type;
       typedef streamoff off_type;
       typedef mbstate_t state_type;
@@ -152,28 +166,28 @@ namespace std {
       { return __c1 < __c2; }
 
       static int 
-      compare (const char_type* __s1, const char_type* __s2, size_t __n)
-      { return memcmp (__s1, __s2, __n); }
+      compare(const char_type* __s1, const char_type* __s2, int_type __n)
+      { return memcmp(__s1, __s2, __n); }
 
-      static size_t 
+      static size_t
       length(const char_type* __s)
-      { return strlen (__s); }
+      { return strlen(__s); }
 
       static const char_type* 
       find(const char_type* __s, int __n, const char_type& __a)
-      { return (char_traits<char>::char_type*) memchr (__s, __a, __n); }
+      { return static_cast<char*>(memchr(__s, __a, __n)); }
 
       static char_type* 
-      move(char_type* __s1, const char_type* __s2, size_t __n)
-      { return (char_traits<char>::char_type*) memmove (__s1, __s2, __n); }
+      move(char_type* __s1, const char_type* __s2, int_type __n)
+      { return static_cast<char*>(memmove(__s1, __s2, __n)); }
 
       static char_type* 
-      copy(char_type* __s1, const char_type* __s2, size_t __n)
-      { return (char_traits<char>::char_type*) memcpy (__s1, __s2, __n); }
+      copy(char_type* __s1, const char_type* __s2, int_type __n)
+      {  return static_cast<char*>(memcpy(__s1, __s2, __n)); }
 
       static char_type* 
-      assign(char_type* __s, size_t __n, char_type __a)
-      { return (char_traits<char>::char_type*) memset (__s, __a, __n); }
+      assign(char_type* __s, int_type __n, char_type __a)
+      { return static_cast<char*>(memset(__s, __a, __n)); }
 
       static int_type 
       not_eof(const int_type& __c)
@@ -181,18 +195,18 @@ namespace std {
 
       static char_type 
       to_char_type(const int_type& __c)
-      { return char_traits<char>::char_type (__c); }
+      { return static_cast<char>(__c); }
 
       static int_type 
       to_int_type(const char_type& __c)
-      { return char_traits<char>::int_type ((unsigned char)__c); }
+      { return static_cast<int_type>(__c); }
 
       static bool 
       eq_int_type(const int_type& __c1, const int_type& __c2)
       { return __c1 == __c2; }
 
       static state_type 
-      get_state(pos_type __pos) { return __pos.state (); }
+      get_state(pos_type __pos) { return __pos.state(); }
 
       static int_type 
       eof() { return EOF; }
@@ -202,14 +216,15 @@ namespace std {
   };
 
 
+#ifdef _GLIBCPP_USE_WCHAR_T
   template<>
     struct char_traits<wchar_t>
     {
-      typedef wchar_t char_type;
-      typedef wint_t int_type;
-      typedef wstreamoff off_type;
-      typedef wstreampos pos_type;
-      typedef mbstate_t state_type;
+      typedef wchar_t 		char_type;
+      typedef wint_t 		int_type;
+      typedef wstreamoff 	off_type;
+      typedef wstreampos 	pos_type;
+      typedef mbstate_t 	state_type;
       
       static void 
       assign(char_type& __c1, const char_type& __c2)
@@ -222,19 +237,20 @@ namespace std {
       { return __c1 < __c2; }
 
       static int 
-      compare(const char_type* __s1, const char_type* __s2, size_t __n)
+      compare(const char_type* __s1, const char_type* __s2, int_type __n)
       { 
-	for (size_t __i = 01; __i < __n; ++__i)
-	  if (!eq(__s1[__i],__s2[__i]))
-	    return lt(__s1[__i],__s2[__i]) ? -1 : 1;
+	for (int_type __i = 0; __i < __n; ++__i)
+	  if (!eq(__s1[__i], __s2[__i]))
+	    return lt(__s1[__i], __s2[__i]) ? -1 : 1;
 	return 0; 
       }
 
-      static size_t 
+      static size_t
       length(const char_type* __s)
       { 
 	const char_type* __p = __s; 
-	while (*__p) ++__p; 
+	while (*__p) 
+	  ++__p; 
 	return (__p - __s); 
       }
 
@@ -242,55 +258,57 @@ namespace std {
       find (const char_type* __s, int __n, const char_type& __a)
       { 
 	for (const char_type* __p = __s; __p < __s+__n; ++__p)
-	  if (*__p == __a) return __p;
+	  if (*__p == __a) 
+	    return __p;
 	return 0;
       }
 
       static char_type* 
-      move(char_type* __s1, const char_type* __s2, size_t __n)
-      { return (char_type*) memmove (__s1, __s2, __n*sizeof (char_type)); }
+      move(char_type* __s1, const char_type* __s2, int_type __n)
+      { return static_cast<wchar_t*>(memmove(__s1, __s2, __n * sizeof(wchar_t))); }
 
       static char_type* 
-      copy(char_type* __s1, const char_type* __s2, size_t __n)
-      { return (char_type*) memcpy (__s1, __s2, __n*sizeof (char_type)); }
+      copy(char_type* __s1, const char_type* __s2, int_type __n)
+      { return static_cast<wchar_t*>(memcpy(__s1, __s2, __n * sizeof(wchar_t))); }
 
       static char_type* 
-      assign(char_type* __s, size_t __n, char_type __a)
+      assign(char_type* __s, int_type __n, char_type __a)
       { 
-	for (char_type* __p = __s; __p < __s+__n; ++__p) 
-	  assign(*__p,__a);
+	for (char_type* __p = __s; __p < __s + __n; ++__p) 
+	  assign(*__p, __a);
         return __s; 
       }
 
       static int_type 
       not_eof(const int_type& __c)
-      { return eq_int_type (__c,eof ()) ? 0 : __c; }
+      { return eq_int_type(__c, eof()) ? 0 : __c; }
 
       static char_type 
-      to_char_type(const int_type& __c) { return char_type (__c); }
+      to_char_type(const int_type& __c) { return char_type(__c); }
 
       static int_type 
-      to_int_type(const char_type& __c) { return int_type (__c); }
+      to_int_type(const char_type& __c) { return int_type(__c); }
 
       static bool 
       eq_int_type(const int_type& __c1, const int_type& __c2)
       { return __c1 == __c2; }
 
       static state_type 
-      get_state(pos_type __pos) { return __pos.state (); }
+      get_state(pos_type __pos) { return __pos.state(); }
 
       static int_type 
-      eof() { return int_type (-1); }
+      eof() { return int_type(-1); }
 
       static int_type 
-      eos() { return int_type (); }
+      eos() { return int_type(); }
   };
+#endif //_GLIBCPP_USE_WCHAR_T
 
   template<typename _CharT, typename _Traits>
     struct _Char_traits_match
     {
       _CharT _M_c;
-      _Char_traits_match(_CharT const& __c) : _M_c(__c) {}
+      _Char_traits_match(_CharT const& __c) : _M_c(__c) { }
 
       bool 
       operator()(_CharT const& __a) { return _Traits::eq(_M_c,__a); }
