@@ -42,18 +42,19 @@
 
 namespace std {
 
-  void
-  __out_of_range(const char *str)
+  __Named_exception::__Named_exception(const string& __str)
   {
-    throw out_of_range(str);
+    strncpy(_M_name, __str.c_str(), _S_bufsize);
+    _M_name[_S_bufsize - 1] = '\0';
   }
 
+  void
+  __out_of_range(const char *str)
+  { throw out_of_range(str); }
 
   void 
   __length_error(const char *str)
-  {
-    throw length_error(str);
-  }
+  { throw length_error(str); }
 
 } //namespace std
 
