@@ -26,5 +26,14 @@
 // invalidate any other reasons why the executable file might be covered by
 // the GNU General Public License.
 
+#include <bits/c++config.h>
+
+#ifdef _GLIBCPP_USE_WCHAR_T
 #define C wchar_t
 #include "string-inst.cc"
+#endif
+
+
+
+
+
