@@ -28,8 +28,8 @@
  *   You should not attempt to use it directly.
  */
 
-#ifndef _CPP_BITS_STL_ALGO_H
-#define _CPP_BITS_STL_ALGO_H 1
+#ifndef __SGI_STL_INTERNAL_ALGO_H
+#define __SGI_STL_INTERNAL_ALGO_H
 
 #include <bits/stl_heap.h>
 
@@ -333,8 +333,11 @@ _ForwardIter1 search(_ForwardIter1 __first1, _ForwardIter1 __last1,
   // Test for a pattern of length 1.
   _ForwardIter2 __tmp(__first2);
   ++__tmp;
-  if (__tmp == __last2)
-    return find(__first1, __last1, *__first2);
+  if (__tmp == __last2) {
+    while (__first1 != __last1 && !__predicate(*__first1, *__first2))
+      ++__first1;
+    return __first1;    
+  }
 
   // General case.
 
@@ -773,7 +776,10 @@ _RandomAccessIter __rotate(_RandomAccessIter __first,
   _Distance __l = __n - __k;
   _RandomAccessIter __result = __first + (__last - __middle);
 
-  if (__k == __l) {
+  if (__k == 0)
+    return __last;
+
+  else if (__k == __l) {
     swap_ranges(__first, __middle, __middle);
     return __result;
   }
@@ -1083,7 +1089,7 @@ _ForwardIter __stable_partition_adaptive(_ForwardIter __first,
 template <class _ForwardIter, class _Predicate, class _Tp, class _Distance>
 inline _ForwardIter
 __stable_partition_aux(_ForwardIter __first, _ForwardIter __last, 
-                                           _Predicate __pred, _Tp*, _Distance*)
+                       _Predicate __pred, _Tp*, _Distance*)
 {
   _Temporary_buffer<_ForwardIter, _Tp> __buf(__first, __last);
   if (__buf.size() > 0)
@@ -1283,8 +1289,8 @@ void __introsort_loop(_RandomAccessIter __first,
     _RandomAccessIter __cut =
       __unguarded_partition(__first, __last,
                             _Tp(__median(*__first,
-                                        *(__first + (__last - __first)/2),
-                                        *(__last - 1))));
+                                         *(__first + (__last - __first)/2),
+                                         *(__last - 1))));
     __introsort_loop(__cut, __last, (_Tp*) 0, __depth_limit);
     __last = __cut;
   }
@@ -1304,8 +1310,8 @@ void __introsort_loop(_RandomAccessIter __first,
     _RandomAccessIter __cut =
       __unguarded_partition(__first, __last,
                             _Tp(__median(*__first,
-                                        *(__first + (__last - __first)/2),
-                                        *(__last - 1), __comp)),
+                                         *(__first + (__last - __first)/2),
+                                         *(__last - 1), __comp)),
        __comp);
     __introsort_loop(__cut, __last, (_Tp*) 0, __depth_limit, __comp);
     __last = __cut;
@@ -1675,8 +1681,8 @@ void __nth_element(_RandomAccessIter __first, _RandomAccessIter __nth,
     _RandomAccessIter __cut =
       __unguarded_partition(__first, __last,
                             _Tp(__median(*__first,
-                                        *(__first + (__last - __first)/2),
-                                        *(__last - 1))));
+                                         *(__first + (__last - __first)/2),
+                                         *(__last - 1))));
     if (__cut <= __nth)
       __first = __cut;
     else 
@@ -1698,9 +1704,9 @@ void __nth_element(_RandomAccessIter __first, _RandomAccessIter __nth,
     _RandomAccessIter __cut =
       __unguarded_partition(__first, __last,
                             _Tp(__median(*__first,
-                                        *(__first + (__last - __first)/2), 
-                                        *(__last - 1),
-                                        __comp)),
+                                         *(__first + (__last - __first)/2), 
+                                         *(__last - 1),
+                                         __comp)),
                             __comp);
     if (__cut <= __nth)
       __first = __cut;
@@ -2887,7 +2893,7 @@ bool is_sorted(_ForwardIter __first, _ForwardIter __last,
 
 __STL_END_NAMESPACE
 
-#endif /* _CPP_BITS_STL_ALGO_H */
+#endif /* __SGI_STL_INTERNAL_ALGO_H */
 
 // Local Variables:
 // mode:C++
