@@ -46,7 +46,7 @@
 #ifdef __STL_USE_NEW_IOSTREAMS 
 #include <iosfwd>
 #else /* __STL_USE_NEW_IOSTREAMS */
-#include <bits/std_iostream.h>
+#include <bits/std_iosfwd.h>
 #endif /* __STL_USE_NEW_IOSTREAMS */
 
 #include <bits/stl_iterator_base.h>
