@@ -35,6 +35,9 @@
 //   function.
 // * __STL_STATIC_TEMPLATE_MEMBER_BUG: defined if the compiler can't handle
 //   static members of template classes.
+// * __STL_STATIC_CONST_INIT_BUG: defined if the compiler can't handle a
+//   constant-initializer in the declaration of a static const data member
+//   of integer type.  (See section 9.4.2, paragraph 4, of the C++ standard.)
 // * __STL_CLASS_PARTIAL_SPECIALIZATION: defined if the compiler supports
 //   partial specialization of template classes.
 // * __STL_PARTIAL_SPECIALIZATION_SYNTAX: defined if the compiler 
@@ -67,7 +70,8 @@
 // * __STL_NO_EXCEPTION_HEADER: defined if the compiler does not have a
 //   standard-conforming header <exception>.
 // * __STL_NO_BAD_ALLOC: defined if the compiler does not have a <new>
-//   header, or if <new> does not contain a bad_alloc class
+//   header, or if <new> does not contain a bad_alloc class.  If a bad_alloc
+//   class exists, it is assumed to be in namespace std.
 // * __STL_SGI_THREADS: defined if this is being compiled for an SGI IRIX
 //   system in multithreaded mode, using native SGI threads instead of 
 //   pthreads.
@@ -75,6 +79,9 @@
 //   compiler in multithreaded mode.
 // * __STL_PTHREADS: defined if we should use portable pthreads
 //   synchronization.
+// * __STL_UITHREADS: defined if we should use UI / solaris / UnixWare threads
+//   synchronization.  UIthreads are similar to pthreads, but are based 
+//   on an earlier version of the Posix threads standard.
 // * __STL_LONG_LONG if the compiler has long long and unsigned long long
 //   types.  (They're not in the C++ standard, but they are expected to be 
 //   included in the forthcoming C9X standard.)
@@ -97,6 +104,8 @@
 // * __STL_ASSERTIONS: if defined, then enable runtime checking through the
 //   __stl_assert macro.
 // * _PTHREADS: if defined, use Posix threads for multithreading support.
+// * _UITHREADS:if defined, use SCO/Solaris/UI threads for multithreading 
+//   support
 // * _NOTHREADS: if defined, don't use any multithreading support.  
 // * __STL_USE_NEW_IOSTREAMS: if defined, then the STL will use new,
 //   standard-conforming iostreams (e.g. the <iosfwd> header).  If not
@@ -117,11 +126,18 @@
 #     define __STL_PTHREADS
 # endif
 
+# if defined(_UITHREADS) && !defined(_PTHREADS) && !defined(_NOTHREADS)
+#     define __STL_UITHREADS
+# endif
+
 # if defined(__sgi) && !defined(__GNUC__)
 #   include <standards.h>
 #   if !defined(_BOOL)
 #     define __STL_NO_BOOL
 #   endif
+#   if defined(_MIPS_SIM) && _MIPS_SIM == _ABIO32
+#     define __STL_STATIC_CONST_INIT_BUG
+#   endif
 #   if defined(_WCHAR_T_IS_KEYWORD)
 #     define __STL_HAS_WCHAR_T 
 #   endif
@@ -131,7 +147,7 @@
 #   ifdef _PARTIAL_SPECIALIZATION_OF_CLASS_TEMPLATES
 #     define __STL_CLASS_PARTIAL_SPECIALIZATION
 #   endif
-#   if (_COMPILER_VERSION >= 730) && defined(_MIPS_SIM) && _MIPS_SIM != _ABI32
+#   if (_COMPILER_VERSION >= 730) && defined(_MIPS_SIM) && _MIPS_SIM != _ABIO32
 #     define __STL_FUNCTION_TMPL_PARTIAL_ORDER
 #   endif
 #   ifdef _MEMBER_TEMPLATES
@@ -144,10 +160,10 @@
 #   if defined(_STANDARD_C_PLUS_PLUS)
 #     define __STL_EXPLICIT_FUNCTION_TMPL_ARGS
 #   endif
-#   if (_COMPILER_VERSION >= 730) && defined(_MIPS_SIM) && _MIPS_SIM != _ABI32
+#   if (_COMPILER_VERSION >= 730) && defined(_MIPS_SIM) && _MIPS_SIM != _ABIO32
 #     define __STL_MEMBER_TEMPLATE_KEYWORD
 #   endif
-#   if defined(_MIPS_SIM) && _MIPS_SIM != _MIPS_SIM_ABI32
+#   if defined(_MIPS_SIM) && _MIPS_SIM == _ABIO32
 #     define __STL_DEFAULT_CONSTRUCTOR_BUG
 #   endif
 #   if !defined(_EXPLICIT_IS_KEYWORD)
@@ -162,7 +178,8 @@
 #   if (_COMPILER_VERSION < 721)
 #     define __STL_NO_EXCEPTION_HEADER
 #   endif
-#   if _COMPILER_VERSION < 730 || !defined(_STANDARD_C_PLUS_PLUS)
+#   if _COMPILER_VERSION < 730 || !defined(_STANDARD_C_PLUS_PLUS) || \
+      !defined(_NAMESPACES)
 #     define __STL_NO_BAD_ALLOC
 #   endif
 #   if !defined(_NOTHREADS) && !defined(__STL_PTHREADS)
@@ -174,8 +191,47 @@
 #   if _COMPILER_VERSION >= 730 && defined(_STANDARD_C_PLUS_PLUS)
 #     define __STL_USE_NEW_IOSTREAMS
 #   endif
+#   if _COMPILER_VERSION >= 730 && defined(_STANDARD_C_PLUS_PLUS)
+#     define __STL_CAN_THROW_RANGE_ERRORS
+#   endif
+#   if _COMPILER_VERSION >= 730 && defined(_STANDARD_C_PLUS_PLUS)
+#     define __SGI_STL_USE_AUTO_PTR_CONVERSIONS
+#   endif
 # endif
 
+
+/*
+ * Jochen Schlick '1999  - added new #defines (__STL)_UITHREADS (for 
+ *                         providing SCO / Solaris / UI thread support)
+ *                       - added the necessary defines for the SCO UDK 7 
+ *                         compiler (and its template friend behavior)
+ *                       - all UDK7 specific STL changes are based on the 
+ *                         macro __USLC__ being defined
+ */
+// SCO UDK 7 compiler (UnixWare 7x, OSR 5, UnixWare 2x)
+# if defined(__USLC__)
+#     define __STL_HAS_WCHAR_T 
+#     define __STL_CLASS_PARTIAL_SPECIALIZATION
+#     define __STL_PARTIAL_SPECIALIZATION_SYNTAX
+#     define __STL_FUNCTION_TMPL_PARTIAL_ORDER
+#     define __STL_MEMBER_TEMPLATES
+#     define __STL_MEMBER_TEMPLATE_CLASSES
+#     define __STL_USE_EXCEPTIONS
+#     define __STL_HAS_NAMESPACES
+#     define __STL_USE_NAMESPACES
+#     define __STL_LONG_LONG
+#     if defined(_REENTRANT)
+#           define _UITHREADS     /* if      UnixWare < 7.0.1 */
+#           define __STL_UITHREADS
+//   use the following defines instead of the UI threads defines when
+//   you want to use POSIX threads
+//#         define _PTHREADS      /* only if UnixWare >=7.0.1 */
+//#         define __STL_PTHREADS
+#     endif
+# endif
+
+
+
 # ifdef __GNUC__
 #   include <_G_config.h>
 #   define __STL_HAS_WCHAR_T
@@ -186,18 +242,20 @@
 #   else
 #     define __STL_MEMBER_TEMPLATES
 #     define __STL_MEMBER_TEMPLATE_CLASSES
-#     define __STL_CLASS_PARTIAL_SPECIALIZATION // stl_tempbuf.h: 146
+#     define __STL_CLASS_PARTIAL_SPECIALIZATION 
 #     define __STL_PARTIAL_SPECIALIZATION_SYNTAX 
 #     define __STL_FUNCTION_TMPL_PARTIAL_ORDER
+//#     define __STL_CAN_THROW_RANGE_ERRORS // till stdexception sorted out
 #     define __STL_EXPLICIT_FUNCTION_TMPL_ARGS
 #     define __SGI_STL_USE_AUTO_PTR_CONVERSIONS
 #     define __STL_HAS_NAMESPACES
 #     define __STL_USE_NAMESPACES
 #     define __STL_USE_EXCEPTIONS
-#     define __STL_USE_STD_ALLOCATORS //so string_fwd.h will not use def_alloc.
+#     define __STL_USE_STD_ALLOCATORS 
+#     define __USE_MALLOC // As the "underlying allocator"
 //#     define __STL_USE_NEW_IOSTREAMS //990209 bkoz--use standard .h includes.
 #   endif
-    /* glibc pre 2.0 is very buggy. We have to disable thread for it.
+    /* glibc pre 2.0 is very buggy. We have to disable threads for it.
        It should be upgraded to glibc 2.0 or later. */
 // 990209 bkoz disable temporarily untill we have makefile support for
 // --enable-threads
@@ -246,16 +304,42 @@
 #   define __STL_HAS_NAMESPACES
 # endif
 
-# if defined(_MSC_VER)
+// Intel compiler, which uses the EDG front end.
+# if defined(__ICL)
+#   define __STL_LONG_LONG 
+#   define __STL_MEMBER_TEMPLATES
+#   define __STL_MEMBER_TEMPLATE_CLASSES
+#   define __STL_FUNCTION_TMPL_PARTIAL_ORDER
+#   define __STL_CLASS_PARTIAL_SPECIALIZATION
 #   define __STL_NO_DRAND48
+#   define __STL_HAS_NAMESPACES
+#   define __STL_USE_EXCEPTIONS
+#   define __STL_MEMBER_TEMPLATE_KEYWORD
+#   ifdef _CPPUNWIND
+#     define __STL_USE_EXCEPTIONS
+#   endif
+#   ifdef _MT
+#     define __STL_WIN32THREADS
+#   endif
+# endif
+
+// Mingw32, EGCS compiler using the Microsoft C runtime
+# if defined(__MINGW32__)
+#   define __STL_NO_DRAND48
+# endif
+
+// Microsoft compiler.
+# if defined(_MSC_VER) && !defined(__ICL)
+#   define __STL_NO_DRAND48
+#   define __STL_STATIC_CONST_INIT_BUG
 #   define __STL_NEED_TYPENAME
 #   if _MSC_VER < 1100  /* 1000 is version 4.0, 1100 is 5.0, 1200 is 6.0. */
 #     define __STL_NEED_EXPLICIT
 #     define __STL_NO_BOOL
-#     if  _MSC_VER > 1000
-#       include <yvals.h>
-#       define __STL_DONT_USE_BOOL_TYPEDEF
-#     endif
+#   endif
+#   if _MSC_VER > 1000
+#     include <yvals.h>
+#     define __STL_DONT_USE_BOOL_TYPEDEF
 #   endif
 #   define __STL_NON_TYPE_TMPL_PARAM_BUG
 #   define __SGI_STL_NO_ARROW_OPERATOR
@@ -270,13 +354,19 @@
 #     define __STL_PARTIAL_SPECIALIZATION_SYNTAX
 #     define __STL_HAS_NAMESPACES
 #     define __STL_NO_NAMESPACES
+#     define __STL_CAN_THROW_RANGE_ERRORS
+#   endif
+#   if _MSC_VER < 1100
+#     define __STL_NO_EXCEPTION_HEADER
+#     define __STL_NO_BAD_ALLOC
 #   endif
     // Because of a Microsoft front end bug, we must not provide a
-    // namespace qualifier when declaring a front end function.
+    // namespace qualifier when declaring a friend function.
 #   define __STD_QUALIFIER
 # endif
 
 # if defined(__BORLANDC__)
+#   define __STL_NO_BAD_ALLOC
 #   define __STL_NO_DRAND48
 #   define __STL_NEED_TYPENAME
 #   define __STL_LIMITED_DEFAULT_TEMPLATES
@@ -422,8 +512,8 @@
 # define __stl_assert(expr)
 #endif
 
-#if defined(__STL_WIN32_THREADS) || defined(STL_SGI_THREADS) \
-    || defined(__STL_PTHREADS)
+#if defined(__STL_WIN32THREADS) || defined(STL_SGI_THREADS) \
+    || defined(__STL_PTHREADS)  || defined(__STL_UITHREADS)
 #   define __STL_THREADS
 #   define __STL_VOLATILE volatile
 #else
