@@ -839,7 +839,7 @@ public:                         // Erase
   iterator erase(iterator __pos) {
     iterator __next = __pos;
     ++__next;
-    difference_type __index = __pos - _M_start;
+    size_type __index = __pos - _M_start;
     if (__index < (size() >> 1)) {
       copy_backward(_M_start, __pos, __next);
       pop_front();
